#!/soft/gnu-os/bin/perl
use strict;

our $SCRIPT = "GenerateIBIS.pl";
our $VERSION = "0.2";
our $DATE = gmtime;

# *****************************************************************
# Author: Ann On-Yee Lau
# Date: September 9, 2008
# Purpose: To Modify the PIN Section of the IBIS Model to comply with
# Customer's specific designs
#
# Version History
# 09/09/08 0.0 Initial Version with Comments 
# edited by  Masciarelli Thierry
# 09/10/08 0.1 Updated the count. NUMSEC was incorrectly shifted
# edited by  Masciarelli Thierry
# 09/17/08 0.2 Add Vcci option on command line to handle PCI3.3 and PCI5.0 (default is 5.0)
#              A DRC verifies compatibility of I/O standards found in the pin file with VCCI option
# *****************************************************************

my $CONSTVCCI50=5.0;
my $CONSTVCCI33=3.3;

# drcVcciVsModel <Vcci> <model>
sub drcVcciVsModel
{
   my $Vcci = $_[0];
   my $IoModel= $_[1];
   if ( ($Vcci == $CONSTVCCI50) && ($IoModel eq "LVTTL")){
       die " IO standard $IoModel not compatible with Vcci $Vcci \n";
       }
   if ( (($Vcci == $CONSTVCCI33) && ($IoModel eq "TTL"))||
        (($Vcci == $CONSTVCCI33) && ($IoModel eq "CMOS"))  ){
       die " IO standard $IoModel not compatible with Vcci $Vcci \n";
       }
   if( ($IoModel ne "TTL")&&
       ($IoModel ne "LVTTL")&&
       ($IoModel ne "PCI")&&
       ($IoModel ne "CMOS")){
	die " IO standard $IoModel not supported\n Standards supported are LVTTL, PCI , CMOS and TTL\n";
       }	

}


unless ($#ARGV + 1 >= 2) {
    die "Usage: $SCRIPT  <PIN RPT> <IBIS FILE> [VCCI]\n Where VCCI is 3.3 or 5.0. Default is 5.0\n";
}

my $PIN_RPT=$ARGV[0];
my $IBSFILE=$ARGV[1];
my $VCCIOPT=$ARGV[2];


my @array; my $die; my $package; my $design;
my %pinlist; my %modeltype; my %drive;
my %ibismodel;
my %aliasmodel;

#1      CMOS5V_HS            CMOS5V_HS
#2      CMOS5V_LS            CMOS5V_LS
#|
#3      TTL_5V_HS            TTL_5V_HS
#4      TTL_5V_LS            TTL_5V_LS
#|
#5      PCI5V                PCI5V
#6      PCI33V               PCI33V
#|
#7      LVTTL_HS             LVTTL_HS
#8      LVTTL_LS             LVTTL_LS

$ibismodel{"LVTTL"}{"Low"}="LVTTL_LS";
$ibismodel{"LVTTL"}{"High"}="LVTTL_HS";
$ibismodel{"TTL"}{"Low"}="TTL_5V_LS";
$ibismodel{"TTL"}{"High"}="TTL_5V_HS";
$ibismodel{"CMOS"}{"Low"}="CMOS5V_LS";
$ibismodel{"CMOS"}{"High"}="CMOS5V_HS";


my $NUMSEC;
open PIN, "$PIN_RPT" || die "Cannot Open PIN REPORT $PIN_RPT\n";
open IBS, "$IBSFILE" || die "Cannot Open IBIS FILE $IBSFILE\n";

if ($VCCIOPT eq ""  ){
	$VCCIOPT=$CONSTVCCI50;	
      print "Warning: Vcci not specified\n Assuming default Vcci = $CONSTVCCI50\n";
    }

if ( $VCCIOPT == $CONSTVCCI50 ){	
     $ibismodel{"PCI"}{"High"}="PCI5V";
   }
elsif ( $VCCIOPT == $CONSTVCCI33 ){
     $ibismodel{"PCI"}{"High"}="PCI33V";
   }
else {
      die "Vcci $VCCIOPT not supported\nVcci supported are $CONSTVCCI50 or $CONSTVCCI33\n";
    }

while (my $line = <PIN>){
	if ($line =~ /^Die/){
		$line=~ s/\s//g;
		@array = split("Die", $line);
		$die=$array[1];
        $die=~ s/54//;
	}
	elsif ($line =~ /^Package/){
		@array = split(" ", $line);
		$package=$array[2].$array[1];
	}
    elsif ($line =~ /Design Name/){
        chomp($line);
		@array = split("Design Name\: ", $line);
        $design = $array[1];
    }
    elsif ($line =~ /^Port/){
        @array = split (/\|/, $line);
        $NUMSEC=($#array -1)  ;
	do{
	  $line = <PIN>;
	  if (!($line =~ /^\-/)){
	  @array = split (" ", $line);
          if ($#array<$NUMSEC){
		    $pinlist{$array[1]} = $array[0];
		    $modeltype{$array[1]} = $array[3];
            if ($array[5] =~ /\-/){ $drive{$array[1]} = "High";}
            else{ $drive{$array[1]} = $array[5];}
          }
          else{
		    $pinlist{$array[1]} = $array[0];
		    $modeltype{$array[1]} = $array[4];
            if ($array[6] =~ /\-/){ $drive{$array[1]} = "High";}
            else{ $drive{$array[1]} = $array[6];}
          }
          #DRC on model type vs VCCIOPT
	  if ($modeltype{$array[1]} ne ""){
            &drcVcciVsModel($VCCIOPT, $modeltype{$array[1]});
          }
  	    $aliasmodel{$array[1]}=$ibismodel{$modeltype{$array[1]}}{$drive{$array[1]}};
	  }	
	} until $line =~ /^$/; 
   }
}

close(PIN);

my $temp = $design.".ibs";
my $NEWIBSFILE = "\L$temp";

open NEWIBSFILE, ">$NEWIBSFILE" || die "Cannot Open IBIS FILE $NEWIBSFILE\n";

my $line;
do {
   $line =<IBS>;
   if ($line =~ /\[Date\]/){ print NEWIBSFILE "\[Date\]     $DATE\n";}
   elsif ($line =~ /\[File name\]/){ print NEWIBSFILE "\[File name\]     $NEWIBSFILE\n";}
   else{ print NEWIBSFILE $line;}
} until $line =~ /^\[Package\]/;

$line = <IBS>;
print NEWIBSFILE "$line|\n";

do{
    $line = <IBS>;
    if (($line =~ /$package/) && ($line =~ /$die/)){
    $line =~ s/\|R_pkg/R_pkg/;
    $line =~ s/\|C_pkg/C_pkg/;
    $line =~ s/\|L_pkg/L_pkg/;
    print NEWIBSFILE $line;
    }
} until $line =~ /^\[Pin\]/;

print NEWIBSFILE "|\n";
print NEWIBSFILE $line;
print NEWIBSFILE "|\n";

do{
    $line = <IBS>;
}until $line =~ /^VCCI/;

for my $pin (sort keys %pinlist){
    print NEWIBSFILE "$pin     $pinlist{$pin}    $aliasmodel{$pin}\n";
}


do {
$line = <IBS>;
print NEWIBSFILE $line;
} until $line =~ /^\[End\]$/;
print "$NEWIBSFILE generated with success \n";
